Package: adephylo
Version: 1.1-17
Title: Exploratory Analyses for the Phylogenetic Comparative Method
Authors@R: c(
   person("Stéphane", "Dray", email = "stephane.dray@univ-lyon1.fr", role = "aut", comment = c(ORCID = "0000-0003-0153-1105")),
   person("Thibaut", "Jombart", role = "aut", email = "t.jombart@imperial.ac.uk"),
   person("Anders Ellern", "Bilgrau", role = "ctb", email = "abilgrau@math.aau.dk"),
   person("Aurélie", "Siberchicot", role = c("ctb", "cre"), email = "aurelie.siberchicot@univ-lyon1.fr", comment = c(ORCID = "0000-0002-7638-8318"))
	)
Depends: methods, ade4 (>= 1.7-10)
Imports: phylobase, ape, adegenet
Description: Multivariate tools to analyze comparative data, i.e. a phylogeny and some traits measured for each taxa. The package contains functions to represent comparative data, compute phylogenetic proximities, perform multivariate analysis with phylogenetic constraints and test for the presence of phylogenetic autocorrelation. The package is described in Jombart et al (2010) <doi:10.1093/bioinformatics/btq292>.
License: GPL (>= 2)
LazyLoad: yes
RoxygenNote: 7.3.2
Encoding: UTF-8
URL: https://github.com/adeverse/adephylo
BugReports: https://github.com/adeverse/adephylo/issues
NeedsCompilation: yes
Packaged: 2025-03-14 10:48:51 UTC; siberchicot
Author: Stéphane Dray [aut] (<https://orcid.org/0000-0003-0153-1105>),
  Thibaut Jombart [aut],
  Anders Ellern Bilgrau [ctb],
  Aurélie Siberchicot [ctb, cre]
    (<https://orcid.org/0000-0002-7638-8318>)
Maintainer: Aurélie Siberchicot <aurelie.siberchicot@univ-lyon1.fr>
Repository: CRAN
Date/Publication: 2025-03-14 17:10:10 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-03-18 02:45:57 UTC; unix
Archs: adephylo.so.dSYM
