# Generated by roxygen2: do not edit by hand

S3method(plot,ppca)
S3method(print,ppca)
S3method(scatter,ppca)
S3method(screeplot,ppca)
S3method(summary,ppca)
export(.tipToRoot)
export(abouheif.moran)
export(bullseye)
export(distRoot)
export(distTips)
export(listDD)
export(listTips)
export(me.phylo)
export(moran.idx)
export(orthobasis.phylo)
export(orthogram)
export(ppca)
export(proxTips)
export(sp.tips)
export(table.phylo4d)
export(treePart)
import(ade4)
import(methods)
import(phylobase)
importFrom(adegenet,any2col)
importFrom(adegenet,spectral)
importFrom(adegenet,transp)
importFrom(ape,.PlotPhyloEnv)
importFrom(ape,plot.phylo)
importFrom(grDevices,grey)
importFrom(grDevices,heat.colors)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,dotchart)
importFrom(graphics,layout)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,symbols)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,median)
importFrom(stats,screeplot)
useDynLib(adephylo)
